@@ -34,6 +34,8 @@
 #include <sys/stat.h>
 #include <unistd.h>
 
+#include "oafd-corba-extensions.h"
+
 /*** App-specific servant structures ***/
 
 typedef struct
@@ -111,6 +113,7 @@ static void
 impl_OAF_ObjectDirectory_unregister (impl_POA_OAF_ObjectDirectory * servant,
 				     OAF_ImplementationID iid,
 				     CORBA_Object obj,
+				     OAF_ObjectDirectory_UnregisterType notify,
 				     CORBA_Environment * ev);
 
 /*** epv structures ***/
@@ -220,7 +223,7 @@ update_registry (impl_POA_OAF_ObjectDirectory *servant)
         gboolean must_load;
         int i;
 
-        /* FIXME: we should only reload those directories that have
+        /* FIXME bugzilla.eazel.com 2727: we should only reload those directories that have
          * actually changed instead of reloading all when any has
          * changed. 
          */
@@ -243,8 +246,6 @@ update_registry (impl_POA_OAF_ObjectDirectory *servant)
         }
 
         if (must_load) {
-                puts ("reload");
-
                 servant->attr_servers._buffer =
                         OAF_ServerInfo_load (servant->registry_source_directories,
                                              &servant->attr_servers._length,
@@ -380,6 +381,42 @@ impl_OAF_ObjectDirectory__get_username (impl_POA_OAF_ObjectDirectory *
 	return CORBA_string_dup (g_get_user_name ());
 }
 
+
+
+static CORBA_Object 
+od_get_active_server (impl_POA_OAF_ObjectDirectory * servant,
+                       OAF_ImplementationID iid,
+                       CORBA_Context ctx, CORBA_Environment * ev)
+{
+	CORBA_Object retval;
+        char *display;
+        char *display_iid;
+
+        display = oafd_CORBA_Context_get_value (ctx, "display", NULL, ev);
+        
+        if (display != NULL) {
+                display_iid = g_strconcat (display, ",", iid, NULL);
+                
+                retval = g_hash_table_lookup (servant->active_servers, display_iid);
+
+                g_free (display_iid);
+                
+                if (!CORBA_Object_is_nil (retval, ev)
+                    && !CORBA_Object_non_existent (retval, ev)) {
+                        return CORBA_Object_duplicate (retval, ev);
+                }
+        }
+
+        retval = g_hash_table_lookup (servant->active_servers, iid);
+        
+        if (!CORBA_Object_is_nil (retval, ev)
+            && !CORBA_Object_non_existent (retval, ev)) {
+                return CORBA_Object_duplicate (retval, ev);
+        }
+
+        return CORBA_OBJECT_NIL;
+}
+
 static CORBA_Object
 impl_OAF_ObjectDirectory_activate (impl_POA_OAF_ObjectDirectory * servant,
 				   OAF_ImplementationID iid,
@@ -391,27 +428,31 @@ impl_OAF_ObjectDirectory_activate (impl_POA_OAF_ObjectDirectory * servant,
 	OAF_ServerInfo *si;
 	ODActivationInfo ai;
 
+	retval = CORBA_OBJECT_NIL;
+
         update_registry (servant);
 
+        if (!(flags & OAF_FLAG_PRIVATE)) {
+                retval = od_get_active_server (servant, iid, ctx, ev);
+
+                if (retval != CORBA_OBJECT_NIL) {
+                        return retval;
+                }
+        }
+
+	if (flags & OAF_FLAG_EXISTING_ONLY) {
+		return CORBA_OBJECT_NIL;
+        }
+
 	ai.ac = ac;
 	ai.flags = flags;
 	ai.ctx = ctx;
-	retval = CORBA_OBJECT_NIL;
 
 	si = g_hash_table_lookup (servant->by_iid, iid);
 
-	retval = g_hash_table_lookup (servant->active_servers, iid);
-
-	if (!CORBA_Object_is_nil (retval, ev)
-	    && !CORBA_Object_non_existent (retval, ev)
-	    && !(flags & OAF_FLAG_PRIVATE))
-		return CORBA_Object_duplicate (retval, ev);
-
-	if (flags & OAF_FLAG_EXISTING_ONLY)
-		return CORBA_OBJECT_NIL;
-
-	if (si)
+	if (si) {
 		retval = od_server_activate (si, &ai, servant->self, ev);
+        }
 
 	return retval;
 }
@@ -441,6 +482,16 @@ impl_OAF_ObjectDirectory_register (impl_POA_OAF_ObjectDirectory * servant,
 				   CORBA_Object obj, CORBA_Environment * ev)
 {
 	CORBA_Object oldobj;
+        OAF_ImplementationID actual_iid;
+
+        actual_iid = strrchr (iid, ',');
+
+        if (actual_iid == NULL) {
+                actual_iid = iid;
+        } else {
+                actual_iid++;
+        }
+
 
 	oldobj = g_hash_table_lookup (servant->active_servers, iid);
 
@@ -452,7 +503,7 @@ impl_OAF_ObjectDirectory_register (impl_POA_OAF_ObjectDirectory * servant,
 	}
 
 
-	if (!g_hash_table_lookup (servant->by_iid, iid))
+	if (!g_hash_table_lookup (servant->by_iid, actual_iid))
 		return OAF_REG_NOT_LISTED;
 
 	g_hash_table_insert (servant->active_servers,
@@ -466,7 +517,9 @@ impl_OAF_ObjectDirectory_register (impl_POA_OAF_ObjectDirectory * servant,
 static void
 impl_OAF_ObjectDirectory_unregister (impl_POA_OAF_ObjectDirectory * servant,
 				     OAF_ImplementationID iid,
-				     CORBA_Object obj, CORBA_Environment * ev)
+				     CORBA_Object obj,
+				     OAF_ObjectDirectory_UnregisterType notify,
+				     CORBA_Environment * ev)
 {
 	char *orig_iid;
 	CORBA_Object orig_obj;
